@@ -39,7 +39,7 @@ pub(crate) fn complete_record(acc: &mut Completions, ctx: &CompletionContext) ->
     };
 
     for (field, ty) in missing_fields {
-        acc.add_field(ctx, field, &ty);
+        acc.add_field(ctx, None, field, &ty);
     }
 
     Some(())
@@ -48,44 +48,29 @@ pub(crate) fn complete_record(acc: &mut Completions, ctx: &CompletionContext) ->
 #[cfg(test)]
 mod tests {
     use expect_test::{expect, Expect};
-    use ide_db::helpers::FamousDefs;
 
     use crate::{
-        test_utils::{self, completion_list},
+        tests::{check_edit, filtered_completion_list},
         CompletionKind,
     };
 
     fn check(ra_fixture: &str, expect: Expect) {
-        let actual = completion_list(ra_fixture, CompletionKind::Reference);
+        let actual = filtered_completion_list(ra_fixture, CompletionKind::Reference);
         expect.assert_eq(&actual);
     }
 
     fn check_snippet(ra_fixture: &str, expect: Expect) {
-        let actual = completion_list(
-            &format!("//- /main.rs crate:main deps:core\n{}\n{}", ra_fixture, FamousDefs::FIXTURE),
-            CompletionKind::Snippet,
-        );
+        let actual = filtered_completion_list(ra_fixture, CompletionKind::Snippet);
         expect.assert_eq(&actual);
     }
 
-    fn check_edit(what: &str, ra_fixture_before: &str, ra_fixture_after: &str) {
-        test_utils::check_edit(
-            what,
-            &format!(
-                "//- /main.rs crate:main deps:core{}\n{}",
-                ra_fixture_before,
-                FamousDefs::FIXTURE,
-            ),
-            &(ra_fixture_after.to_owned() + "\n"),
-        );
-    }
-
     #[test]
     fn test_record_literal_field_default() {
         let test_code = r#"
+//- minicore: default
 struct S { foo: u32, bar: usize }
 
-impl core::default::Default for S {
+impl Default for S {
     fn default() -> Self {
         S {
             foo: 0,
@@ -121,9 +106,10 @@ fn process(f: S) {
         check_edit(
             "..Default::default()",
             r#"
+//- minicore: default
 struct S { foo: u32, bar: usize }
 
-impl core::default::Default for S {
+impl Default for S {
     fn default() -> Self {
         S {
             foo: 0,
@@ -142,7 +128,7 @@ fn process(f: S) {
             r#"
 struct S { foo: u32, bar: usize }
 
-impl core::default::Default for S {
+impl Default for S {
     fn default() -> Self {
         S {
             foo: 0,
