@@ -39,8 +39,7 @@ pub struct CompletionItem {
     ///
     /// Typically, replaces `source_range` with new identifier.
     text_edit: TextEdit,
-
-    insert_text_format: InsertTextFormat,
+    is_snippet: bool,
 
     /// What item (struct, function, etc) are we completing.
     kind: Option<CompletionItemKind>,
@@ -145,6 +144,15 @@ pub struct CompletionRelevance {
     /// }
     /// ```
     pub is_local: bool,
+    /// This is set in cases like these:
+    ///
+    /// ```
+    /// (a > b).not$0
+    /// ```
+    ///
+    /// Basically, we want to guarantee that postfix snippets always takes
+    /// precedence over everything else.
+    pub exact_postfix_snippet_match: bool,
 }
 
 #[derive(Debug, Clone, Copy, Eq, PartialEq)]
@@ -195,7 +203,9 @@ impl CompletionRelevance {
         if self.is_local {
             score += 1;
         }
-
+        if self.exact_postfix_snippet_match {
+            score += 100;
+        }
         score
     }
 
@@ -272,12 +282,6 @@ pub(crate) enum CompletionKind {
     Attribute,
 }
 
-#[derive(Debug, PartialEq, Eq, Copy, Clone)]
-pub enum InsertTextFormat {
-    PlainText,
-    Snippet,
-}
-
 impl CompletionItem {
     pub(crate) fn new(
         completion_kind: CompletionKind,
@@ -290,7 +294,7 @@ impl CompletionItem {
             completion_kind,
             label,
             insert_text: None,
-            insert_text_format: InsertTextFormat::PlainText,
+            is_snippet: false,
             detail: None,
             documentation: None,
             lookup: None,
@@ -312,13 +316,13 @@ impl CompletionItem {
         self.source_range
     }
 
-    pub fn insert_text_format(&self) -> InsertTextFormat {
-        self.insert_text_format
-    }
-
     pub fn text_edit(&self) -> &TextEdit {
         &self.text_edit
     }
+    /// Whether `text_edit` is a snippet (contains `$0` markers).
+    pub fn is_snippet(&self) -> bool {
+        self.is_snippet
+    }
 
     /// Short one-line additional information, like a type
     pub fn detail(&self) -> Option<&str> {
@@ -396,7 +400,7 @@ pub(crate) struct Builder {
     import_to_add: Option<ImportEdit>,
     label: String,
     insert_text: Option<String>,
-    insert_text_format: InsertTextFormat,
+    is_snippet: bool,
     detail: Option<String>,
     documentation: Option<Documentation>,
     lookup: Option<String>,
@@ -442,8 +446,8 @@ impl Builder {
         CompletionItem {
             source_range: self.source_range,
             label,
-            insert_text_format: self.insert_text_format,
             text_edit,
+            is_snippet: self.is_snippet,
             detail: self.detail,
             documentation: self.documentation,
             lookup,
@@ -473,7 +477,7 @@ impl Builder {
         _cap: SnippetCap,
         snippet: impl Into<String>,
     ) -> &mut Builder {
-        self.insert_text_format = InsertTextFormat::Snippet;
+        self.is_snippet = true;
         self.insert_text(snippet)
     }
     pub(crate) fn kind(&mut self, kind: impl Into<CompletionItemKind>) -> &mut Builder {
@@ -485,7 +489,7 @@ impl Builder {
         self
     }
     pub(crate) fn snippet_edit(&mut self, _cap: SnippetCap, edit: TextEdit) -> &mut Builder {
-        self.insert_text_format = InsertTextFormat::Snippet;
+        self.is_snippet = true;
         self.text_edit(edit)
     }
     pub(crate) fn detail(&mut self, detail: impl Into<String>) -> &mut Builder {
@@ -605,6 +609,13 @@ mod tests {
                 exact_name_match: true,
                 type_match: Some(CompletionRelevanceTypeMatch::Exact),
                 is_local: true,
+                ..CompletionRelevance::default()
+            }],
+            vec![CompletionRelevance {
+                exact_name_match: false,
+                type_match: None,
+                is_local: false,
+                exact_postfix_snippet_match: true,
             }],
         ];
 
