@@ -330,6 +330,8 @@ fn compute_ref_match(
 
 #[cfg(test)]
 mod tests {
+    use std::cmp;
+
     use expect_test::{expect, Expect};
     use itertools::Itertools;
 
@@ -339,32 +341,25 @@ mod tests {
         CompletionKind, CompletionRelevance,
     };
 
+    #[track_caller]
     fn check(ra_fixture: &str, expect: Expect) {
         let actual = do_completion(ra_fixture, CompletionKind::Reference);
         expect.assert_debug_eq(&actual);
     }
 
+    #[track_caller]
     fn check_relevance(ra_fixture: &str, expect: Expect) {
-        fn display_relevance(relevance: CompletionRelevance) -> String {
-            let relevance_factors = vec![
-                (relevance.type_match == Some(CompletionRelevanceTypeMatch::Exact), "type"),
-                (
-                    relevance.type_match == Some(CompletionRelevanceTypeMatch::CouldUnify),
-                    "type_could_unify",
-                ),
-                (relevance.exact_name_match, "name"),
-                (relevance.is_local, "local"),
-            ]
-            .into_iter()
-            .filter_map(|(cond, desc)| if cond { Some(desc) } else { None })
-            .join("+");
+        check_relevance_for_kinds(&[CompletionKind::Reference], ra_fixture, expect)
+    }
 
-            format!("[{}]", relevance_factors)
-        }
+    #[track_caller]
+    fn check_relevance_for_kinds(kinds: &[CompletionKind], ra_fixture: &str, expect: Expect) {
+        let mut actual = get_all_items(TEST_CONFIG, ra_fixture);
+        actual.retain(|it| kinds.contains(&it.completion_kind));
+        actual.sort_by_key(|it| cmp::Reverse(it.relevance().score()));
 
-        let actual = get_all_items(TEST_CONFIG, ra_fixture)
+        let actual = actual
             .into_iter()
-            .filter(|it| it.completion_kind == CompletionKind::Reference)
             .flat_map(|it| {
                 let mut items = vec![];
 
@@ -384,6 +379,24 @@ mod tests {
             .collect::<String>();
 
         expect.assert_eq(&actual);
+
+        fn display_relevance(relevance: CompletionRelevance) -> String {
+            let relevance_factors = vec![
+                (relevance.type_match == Some(CompletionRelevanceTypeMatch::Exact), "type"),
+                (
+                    relevance.type_match == Some(CompletionRelevanceTypeMatch::CouldUnify),
+                    "type_could_unify",
+                ),
+                (relevance.exact_name_match, "name"),
+                (relevance.is_local, "local"),
+                (relevance.exact_postfix_snippet_match, "snippet"),
+            ]
+            .into_iter()
+            .filter_map(|(cond, desc)| if cond { Some(desc) } else { None })
+            .join("+");
+
+            format!("[{}]", relevance_factors)
+        }
     }
 
     #[test]
@@ -528,6 +541,7 @@ fn main() { let _: m::Spam = S$0 }
                                 Exact,
                             ),
                             is_local: false,
+                            exact_postfix_snippet_match: false,
                         },
                         trigger_call_info: true,
                     },
@@ -556,6 +570,7 @@ fn main() { let _: m::Spam = S$0 }
                                 Exact,
                             ),
                             is_local: false,
+                            exact_postfix_snippet_match: false,
                         },
                     },
                     CompletionItem {
@@ -649,6 +664,7 @@ fn foo() { A { the$0 } }
                                 CouldUnify,
                             ),
                             is_local: false,
+                            exact_postfix_snippet_match: false,
                         },
                     },
                 ]
@@ -910,9 +926,9 @@ fn test(bar: u32) { }
 fn foo(s: S) { test(s.$0) }
 "#,
             expect![[r#"
-                fd foo []
                 fd bar [type+name]
                 fd baz [type]
+                fd foo []
             "#]],
         );
     }
@@ -926,9 +942,9 @@ struct B { x: (), y: f32, bar: u32 }
 fn foo(a: A) { B { bar: a.$0 }; }
 "#,
             expect![[r#"
-                fd foo []
                 fd bar [type+name]
                 fd baz [type]
+                fd foo []
             "#]],
         )
     }
@@ -956,9 +972,9 @@ fn f(foo: i64) {  }
 fn foo(a: A) { f(B { bar: a.$0 }); }
 "#,
             expect![[r#"
-                fd foo []
                 fd bar [type+name]
                 fd baz [type]
+                fd foo []
             "#]],
         );
     }
@@ -1004,9 +1020,9 @@ fn bar() -> u8 { 0 }
 fn f() { A { bar: b$0 }; }
 "#,
             expect![[r#"
+                fn bar() [type+name]
                 fn baz() [type]
                 st A []
-                fn bar() [type+name]
                 fn f() []
             "#]],
         );
@@ -1329,9 +1345,9 @@ fn foo() {
 }
 "#,
             expect![[r#"
+                lc foo [type+local]
                 ev Foo::A(…) [type_could_unify]
                 ev Foo::B [type_could_unify]
-                lc foo [type+local]
                 en Foo []
                 fn baz() []
                 fn bar() []
@@ -1339,4 +1355,44 @@ fn foo() {
             "#]],
         );
     }
+
+    #[test]
+    fn postfix_completion_relevance() {
+        check_relevance_for_kinds(
+            &[CompletionKind::Postfix, CompletionKind::Magic],
+            r#"
+mod ops {
+    pub trait Not {
+        type Output;
+        fn not(self) -> Self::Output;
+    }
+
+    impl Not for bool {
+        type Output = bool;
+        fn not(self) -> bool { if self { false } else { true }}
+    }
+}
+
+fn main() {
+    let _: bool = (9 > 2).not$0;
+}
+"#,
+            expect![[r#"
+                sn not [snippet]
+                me not() (ops::Not) [type_could_unify]
+                sn if []
+                sn while []
+                sn ref []
+                sn refm []
+                sn match []
+                sn box []
+                sn ok []
+                sn err []
+                sn some []
+                sn dbg []
+                sn dbgr []
+                sn call []
+            "#]],
+        );
+    }
 }
