@@ -1,10 +1,11 @@
 //! Renderer for macro invocations.
 
-use hir::{Documentation, HasSource};
+use hir::HasSource;
 use ide_db::SymbolKind;
 use syntax::display::macro_label;
 
 use crate::{
+    context::CallKind,
     item::{CompletionItem, CompletionKind, ImportEdit},
     render::RenderContext,
 };
@@ -12,7 +13,7 @@ use crate::{
 pub(crate) fn render_macro<'a>(
     ctx: RenderContext<'a>,
     import_to_add: Option<ImportEdit>,
-    name: String,
+    name: hir::Name,
     macro_: hir::MacroDef,
 ) -> Option<CompletionItem> {
     let _p = profile::span("render_macro");
@@ -24,13 +25,14 @@ struct MacroRender<'a> {
     ctx: RenderContext<'a>,
     name: String,
     macro_: hir::MacroDef,
-    docs: Option<Documentation>,
+    docs: Option<hir::Documentation>,
     bra: &'static str,
     ket: &'static str,
 }
 
 impl<'a> MacroRender<'a> {
-    fn new(ctx: RenderContext<'a>, name: String, macro_: hir::MacroDef) -> MacroRender<'a> {
+    fn new(ctx: RenderContext<'a>, name: hir::Name, macro_: hir::MacroDef) -> MacroRender<'a> {
+        let name = name.to_string();
         let docs = ctx.docs(macro_);
         let docs_str = docs.as_ref().map_or("", |s| s.as_str());
         let (bra, ket) = guess_macro_braces(&name, docs_str);
@@ -67,7 +69,8 @@ impl<'a> MacroRender<'a> {
     }
 
     fn needs_bang(&self) -> bool {
-        self.ctx.completion.use_item_syntax.is_none() && !self.ctx.completion.is_macro_call
+        !self.ctx.completion.in_use_tree()
+            && !matches!(self.ctx.completion.path_call_kind(), Some(CallKind::Mac))
     }
 
     fn label(&self) -> String {
@@ -130,7 +133,7 @@ fn guess_macro_braces(macro_name: &str, docs: &str) -> (&'static str, &'static s
 
 #[cfg(test)]
 mod tests {
-    use crate::test_utils::check_edit;
+    use crate::tests::check_edit;
 
     #[test]
     fn dont_insert_macro_call_parens_unncessary() {
@@ -177,7 +180,7 @@ fn main() { frobnicate!(); }
 /// ```
 macro_rules! vec { () => {} }
 
-fn fn main() { v$0 }
+fn main() { v$0 }
 "#,
             r#"
 /// Creates a [`Vec`] containing the arguments.
@@ -190,7 +193,7 @@ fn fn main() { v$0 }
 /// ```
 macro_rules! vec { () => {} }
 
-fn fn main() { vec![$0] }
+fn main() { vec![$0] }
 "#,
         );
 
